require_relative 'spec_helper'


describe Handler do
  let(:app) {{ object: {api_key: 'an api_key'}.to_json, option: {user: false,app: true}.to_json }}
  let(:app_session) {{object: {api_key: 'an api_key'}.to_json,token: "ncidjdojdojfdocv6777",option: {user: false,app: true}.to_json} }
  let(:redis) { Redis.current}
  let(:user) {{ object: {login: 'login',password: 'password'}, option: {user: true,app: false} }}
  let(:fake_app_session) {{api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b',token: 'token1'} }
 let(:fake_app_session2) {{api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b',token: 'token2'} }

   before (:each) do
      redis.del('applications')
      redis.del('users')
      redis.sadd('applications',{api_key: 'an api_key'}.to_json)
      redis.sadd('applications',{api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b'}.to_json)
   end


  describe '.connect_app' do

   before (:each) do
      redis.sadd('applications',app[:object])
      redis.sadd('users',user[:object].to_json)
      redis.sadd('applications_sessions',{api_key: 'an api_key',token: "ncidjdojdojfdocv6777"}.to_json)
      redis.sadd('applications_sessions',fake_app_session.to_json)
      redis.sadd('applications_sessions',fake_app_session2.to_json)
   end

    context "When an application attempts to log in" do
      it { expect(Handler.connect_app(app)).to be_a String }
    end

    context "When a inexistant app attemps to log in" do
      bad_app = {object: {api_key: '013'}.to_json,option: {user: false,app: true}.to_json}
      it { expect(Handler.connect_app(bad_app)).to eq false }
    end

    context "When the parameter is not a hash" do
      it { expect(Handler.connect_app("coucou")).to eq false }
    end

    context "When the parmeter has not valid keys" do
      invalid_param = {obj: {api_key: 'ap'}.to_json,opti: {user: false,app: true}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }
    end

    context "When the parameter's values are not hash" do
      invalid_param = {object: "user",option: {user: false,app: true}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }
    end

    context "When the parameter's values nil or empty" do
      invalid_param = {object: {}.to_json,option: {user: false,app: true}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }

      invalid_param = {oject: nil,option: nil}
      it { expect(Handler.connect_app(invalid_param)).to eq false }
    end

    context "When the option is invalid" do
      invalid_param = {object: {api_key: 'a api_key'}.to_json,option: {use: false,ap: true}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }

      invalid_param = {object: {api_key: 'a api'}.to_json,option: {user: false,app: false}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }

      invalid_param = {object: {api_key: 'a api'}.to_json,option: {user: true,app: true}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }

      invalid_param = {object: {api_key: 'a api'}.to_json,option: {user: 'cou',app: true}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }

      invalid_param = {object: {api_key: 'a api'}.to_json,option: {user: false,app: nil}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }
    end

    context "When the object is invalid" do
      invalid_param = {object: {ap_key: 'a api'}.to_json,option: {user: false,app: true}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }

      invalid_param = {object: {api_key: nil}.to_json,option: {user: false,app: true}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }

      invalid_param = {object: {api_key: ''}.to_json,option: {user: false,app: true}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }

      invalid_param = {object: {api_key: 1}.to_json,option: {user: false,app: true}.to_json}
      it { expect(Handler.connect_app(invalid_param)).to eq false }
    end

  end

  describe '.load_data' do
    context "When data are successfully load" do
     populate_engine
     it {expect(Handler.load_data(params_to_load1)).to eq true }
     it {expect(Handler.load_data(params_to_load2)).to eq true }
    end
  end

  describe '.similar_users' do
    context "When similar users are found" do
      it {expect(Handler.similar_users(params_user_based)).to_not eq false }
    end
    context "When similar_users parameter are not valid" do
      param = {engine_name: 1,api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b',id: 1,nb_wanted: 2}
      it {expect(Handler.similar_users(param)).to eq false }
      param = {engine_name: "eng869",api_key: 2,id: 1,nb_wanted: 2}
      it {expect(Handler.similar_users(param)).to eq false }
      param = {engine_name: "eng869",api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b',id: '1',nb_wanted: 2}
      it {expect(Handler.similar_users(param)).to eq false }
      param = {engine_name: 1,api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b',id: 1,nb_wanted: '2'}
      it {expect(Handler.similar_users(param)).to eq false }
    end
  end

  describe '.similar_items' do
    context "When similar items are found" do
      it {expect(Handler.similar_items(params_item_based)).to_not eq false }
    end
    context "When similar_users parameter are not valid" do
      param = {engine_name: 1,api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b',id: 1,nb_wanted: 2}
      it {expect(Handler.similar_items(param)).to eq false }
      param = {engine_name: "eng111",api_key: 2,id: 1,nb_wanted: 2}
      it {expect(Handler.similar_items(param)).to eq false }
      param = {engine_name: "eng111",api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b',id: '1',nb_wanted: 2}
      it {expect(Handler.similar_items(param)).to eq false }
      param = {engine_name: 1,api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b',id: 1,nb_wanted: '2'}
      it {expect(Handler.similar_items(param)).to eq false }
    end
  end

  describe '.recommend' do
    context "When recommendations are sent" do
      it {expect(Handler.recommend(params_item_based)).to_not eq false }
    end

    context "When similar_users parameter are not valid" do
      param = {engine_name: 1,api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b',id: 1,nb_wanted: 2}
      it {expect(Handler.recommend(param)).to eq false }
      param = {engine_name: "eng111",api_key: 2,id: 1,nb_wanted: 2}
      it {expect(Handler.recommend(param)).to eq false }
      param = {engine_name: "eng111",api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b',id: '1',nb_wanted: 2}
      it {expect(Handler.recommend(param)).to eq false }
      param = {engine_name: 1,api_key: '926eb0b0-e99d-0131-7e06-080027ab4d7b',id: 1,nb_wanted: '2'}
      it {expect(Handler.recommend(param)).to eq false }
    end
  end

  describe '.valid_session' do
    context "when app's session is found" do
      it{expect(Handler.valid_session(app_session)).to eq true}
    end
  end
end
